from setuptools import setup

setup(
    name='xbutils',
    version='0.1.1',
    packages=['xbutils'],
    url='https://github.com/exbee49/xbutils',
    license='WTFPL',
    author='xb49',
    author_email='',
    description='Python utilities',
    long_description="Some Python utility classes",
    long_description_content_type="text/plain",

    classifiers=[
        "Development Status :: 4 - Beta",
        # "license :: Public Domain :: WTFPLv2",
        "Programming Language :: Python :: 3 :: Only"
    ]
)
